@@ -1,5 +1,5 @@
 /* SAMHAIN file system integrity testing                                   */
-/* Copyright (C) 2003 Rainer Wichmann                                      */
+/* Copyright (C) 2003,2005 Rainer Wichmann                                 */
 /*                                                                         */
 /*  This program is free software; you can redistribute it                 */
 /*  and/or modify                                                          */
@@ -23,6 +23,10 @@
  * #define SH_DEBUG_SOCKET
  */
 
+#if defined(SH_WITH_SERVER) && defined(__linux__)
+#define _GNU_SOURCE
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -49,27 +53,48 @@ void sh_socket_server_cmd(const char * srvcmd)
     {
       SL_RET0(_("sh_socket_server_cmd"));
     }
-  if ((srvcmd[0] == 'S') || (srvcmd[1] == 'T') || 
-      (srvcmd[2] == 'O') || (srvcmd[3] == 'P'))
+  if ((srvcmd[0] == 'S') && (srvcmd[1] == 'T') && 
+      (srvcmd[2] == 'O') && (srvcmd[3] == 'P'))
     {
       TPT((0, FIL__, __LINE__, _("msg=<stop command from server>\n")));
 #ifdef SIGQUIT
       raise(SIGQUIT);
 #else
-      TPT((0, FIL__, __LINE__, _("msg=<sigquit not defined>\n")));
+      sig_terminate       = 1;
+      ++sig_raised;
 #endif
     } 
-  else if ((srvcmd[0] == 'R') || (srvcmd[1] == 'E') || 
-	   (srvcmd[2] == 'L') || (srvcmd[3] == 'O') ||
-	   (srvcmd[4] == 'A') || (srvcmd[5] == 'D'))
+  else if ((srvcmd[0] == 'R') && (srvcmd[1] == 'E') &&
+	   (srvcmd[2] == 'L') && (srvcmd[3] == 'O') &&
+	   (srvcmd[4] == 'A') && (srvcmd[5] == 'D'))
     {
       TPT((0, FIL__, __LINE__, _("msg=<reload command from server>\n")));
 #ifdef SIGHUP
       raise(SIGHUP);
 #else
-      TPT((0, FIL__, __LINE__, _("msg=<sighup not defined>\n")));
+      sig_config_read_again = 1;
+      ++sig_raised;
 #endif
     }
+  else if ((srvcmd[0] == 'S') && (srvcmd[1] == 'C') &&
+	   (srvcmd[2] == 'A') && (srvcmd[3] == 'N'))
+    {
+      TPT((0, FIL__, __LINE__, _("msg=<scan command from server>\n")));
+      if (sh.flag.isdaemon == ON) 
+	{ 
+#ifdef SIGTTOU
+	  raise(SIGTTOU);
+#else
+	  sig_force_check = 1;
+	  ++sig_raised;
+#endif
+	} 
+      else 
+	{
+	  sig_force_check = 1;
+	  ++sig_raised;
+	}
+    }
   else
     {
       sh_error_handle(SH_ERR_INFO, FIL__, __LINE__, 0, MSG_E_SUBGEN,
@@ -91,6 +116,8 @@ void sh_socket_server_cmd(const char * srvcmd)
 #include <unistd.h>
 #include <fcntl.h>
 
+#include <time.h>
+
 #include <sys/socket.h>
 #include <sys/un.h>
 
@@ -108,8 +135,7 @@ void sh_socket_server_cmd(const char * srvcmd)
 
 int    pf_unix_fd  = -1;
 static char * sh_sockname = NULL;
-/* static char   sh_sockpass[29]; */
-static char   sh_sockpass_real[15];
+static char   sh_sockpass_real[SOCKPASS_MAX+1];
 
 struct socket_cmd {
   char cmd[SH_MAXMSGLEN];
@@ -130,27 +156,136 @@ struct socket_cmd {
 #define AF_FILE AF_UNIX
 #endif
 
-static struct socket_cmd * cmdlist = NULL;
-static struct socket_cmd * runlist = NULL;
+static struct socket_cmd * cmdlist    = NULL;
+static struct socket_cmd * runlist    = NULL;
 
 static int    sh_socket_flaguse = S_FALSE;
 static int    sh_socket_flaguid = 0;
 
 #include "sh_utils.h"
 
-char * sh_get_sockpass ()
+
+struct reload_cmd {
+  char          clt[SH_MAXMSGLEN];
+  time_t        cti;
+  struct reload_cmd * next;
+};
+static struct reload_cmd * reloadlist = NULL;
+
+void sh_socket_add2reload (const char * clt)
 {
-  int j = 0;
-  while (skey->sh_sockpass[2*j] != '\0')
+  struct reload_cmd  * new = reloadlist;
+
+  while (new)
+    {
+      if (0 == sl_strcmp(new->clt, clt))
+	{
+#ifdef SH_DEBUG_SOCKET
+	  fprintf(stderr, "add2reload: time reset for %s\n", clt);
+#endif
+	  sl_strlcpy (new->clt, clt, SH_MAXMSGLEN);
+	  new->cti = time(NULL);
+	  return;
+	}
+      new = new->next;
+    }
+
+  new = SH_ALLOC(sizeof(struct reload_cmd));
+#ifdef SH_DEBUG_SOCKET
+  fprintf(stderr, "add2reload: time set for %s\n", clt);
+#endif
+  sl_strlcpy (new->clt, clt, SH_MAXMSGLEN);
+  new->cti = time(NULL);
+
+  new->next    = reloadlist;
+  reloadlist   = new;
+
+  return;
+}
+
+#include "zAVLTree.h"
+#include "sh_html.h"
+#include "sh_tools.h"
+static void sh_socket_add2list (struct socket_cmd * in);
+
+static void sh_socket_probe4reload (void)
+{
+  struct reload_cmd  * new;
+  struct socket_cmd    cmd;
+
+  zAVLCursor avlcursor;
+  client_t * item;
+  extern zAVLTree * all_clients;
+
+  char     * file;
+  unsigned long dummy;
+  struct stat buf;
+
+#ifdef SH_DEBUG_SOCKET
+  fprintf(stderr, "PROBE\n");
+#endif
+
+  for (item = (client_t *) zAVLFirst(&avlcursor, all_clients); item;
+       item = (client_t *) zAVLNext(&avlcursor))
+    {
+#ifdef SH_DEBUG_SOCKET
+      fprintf(stderr, "%s %d\n", item->hostname, (int)item->status_now);
+#endif
+
+      if (item->status_now != CLT_INACTIVE)
+	{
+	  file = get_client_conf_file (item->hostname, &dummy);
+
+#ifdef SH_DEBUG_SOCKET
+	  fprintf(stderr, "%s\n", file);
+#endif
+	  if (0 == stat (file, &buf))
+	    {
+	      new = reloadlist;
+	      while (new)
+		{
+#ifdef SH_DEBUG_SOCKET
+		  fprintf(stderr, "%s <> %s\n", new->clt, item->hostname);
+#endif
+		  if (0 == sl_strcmp(new->clt, item->hostname))
+		    {
+#ifdef SH_DEBUG_SOCKET
+		      fprintf(stderr, "%lu <> %lu\n", 
+			      (unsigned long) buf.st_mtime, 
+			      (unsigned long)new->cti);
+#endif
+		      if (buf.st_mtime > new->cti)
+			{
+			  /* reload */
+			  sl_strlcpy(cmd.cmd, _("RELOAD"),    SH_MAXMSGLEN);
+			  sl_strlcpy(cmd.clt, item->hostname, SH_MAXMSGLEN);
+			  sh_socket_add2list (&cmd);
+			}
+		      break;
+		    }
+		  new = new->next;
+		}
+	    }
+	}
+    }
+  return;
+}
+
+char * sh_get_sockpass (void)
+{
+  size_t j = 0;
+
+  while (skey->sh_sockpass[2*j] != '\0' && j < sizeof(sh_sockpass_real))
     {
       sh_sockpass_real[j] = skey->sh_sockpass[2*j];
       ++j;
     }
   sh_sockpass_real[j] = '\0';
+
   return sh_sockpass_real;
 }
 
-void sh_set_sockpass ()
+void sh_set_sockpass (void)
 {
   int j;
   for (j = 0; j < 15; ++j)
@@ -159,7 +294,7 @@ void sh_set_sockpass ()
     }
 }
 
-int sh_socket_use (char * c)
+int sh_socket_use (const char * c)
 {
   return sh_util_flagval(c, &sh_socket_flaguse);
 }
@@ -204,7 +339,7 @@ int sh_socket_remove ()
 
 #endif
 
-int sh_socket_uid (char * c)
+int sh_socket_uid (const char * c)
 {
   uid_t val = (uid_t) strtol (c, (char **)NULL, 10);
   sh_socket_flaguid = val;
@@ -216,7 +351,7 @@ int sh_socket_uid (char * c)
   return 0;
 }
 
-int sh_socket_password (char * c)
+int sh_socket_password (const char * c)
 {
 #if defined(NEED_PASSWORD_AUTH)
   int j, i;
@@ -224,10 +359,10 @@ int sh_socket_password (char * c)
 #define LCG(n) ((69069 * n) & 0xffffffffUL)
 
   i = sl_strlen(c);
-  if (i > 14) {
+  if (i > SOCKPASS_MAX) {
     return -1;
   }
-  for (j = 0; j < 29; ++j)
+  for (j = 0; j < (2*SOCKPASS_MAX+1); ++j)
     {
       skey->sh_sockpass[j] = '\0';
     }
@@ -256,6 +391,7 @@ int sh_socket_open_int ()
   socklen_t    optval = 1;
 #endif
   struct stat buf;
+  char errbuf[SH_ERRBUF_SIZE];
   
   SL_ENTER(_("sh_socket_open_int"));
 
@@ -267,25 +403,31 @@ int sh_socket_open_int ()
   if (sh_sockname == NULL)
     {
       size = sl_strlen(DEFAULT_PIDDIR) + 1 + sl_strlen(SH_INSTALL_NAME) + 6;
-      sh_sockname = SH_ALLOC(size);
+      sh_sockname = SH_ALLOC(size); /* compile-time constant */
       sl_strlcpy(sh_sockname, DEFAULT_PIDDIR, size);
       sl_strlcat(sh_sockname, "/", size);
       sl_strlcat(sh_sockname, SH_INSTALL_NAME, size);
       sl_strlcat(sh_sockname, _(".sock"), size);
     }
 
+  if (0 != sh_unix_check_piddir (sh_sockname))
+    {
+      SH_FREE(sh_sockname);
+      SL_RETURN((-1),_("sh_socket_open_int"));
+    }
+
   pf_unix_fd = socket (PF_UNIX, SOCK_STREAM, 0);
   if ((pf_unix_fd) < 0)
     {
       sh_error_handle ((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		       sh_error_message (errno), 
+		       sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		       _("sh_socket_open_int: socket"));
       SL_RETURN( (-1), _("sh_socket_open_int"));
     }
 
   if (sizeof(name.sun_path) < (1 + sl_strlen(sh_sockname)))
     {
-      close(pf_unix_fd); pf_unix_fd = -1;
+      sl_close_fd(FIL__, __LINE__, pf_unix_fd); pf_unix_fd = -1;
       sh_error_handle ((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
 		       _("PID dir path too long"), 
 		       _("sh_socket_open_int"));
@@ -293,7 +435,7 @@ int sh_socket_open_int ()
     }
 
   name.sun_family = AF_FILE;
-  strcpy (name.sun_path, sh_sockname);
+  sl_strlcpy (name.sun_path, sh_sockname, sizeof(name.sun_path));
 
   size = (offsetof (struct sockaddr_un, sun_path)
           + strlen (name.sun_path) + 1);
@@ -302,14 +444,14 @@ int sh_socket_open_int ()
 
   if (flags == 0)
     {
-      sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+      sh_error_handle(SH_ERR_INFO, FIL__, __LINE__, -1, MSG_E_SUBGEN,
 		      _("Socket exists, trying to unlink it"), 
 		      _("sh_socket_open_int"));
       if (sh_socket_remove() < 0) 
 	{
-	  close(pf_unix_fd); pf_unix_fd = -1;
+	  sl_close_fd(FIL__, __LINE__, pf_unix_fd); pf_unix_fd = -1;
 	  sh_error_handle ((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
-			   _("Unlink failed, maybe path not trusted"), 
+			   _("Unlink of socket failed, maybe path not trusted"), 
 			   _("sh_socket_open_int"));
 	  SL_RETURN( (-1), _("sh_socket_open_int"));
 	}
@@ -317,9 +459,9 @@ int sh_socket_open_int ()
 
   if (bind ((pf_unix_fd), (struct sockaddr *) &name, size) < 0)
     {
-      close(pf_unix_fd); pf_unix_fd = -1;
+      sl_close_fd(FIL__, __LINE__, pf_unix_fd); pf_unix_fd = -1;
       sh_error_handle ((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		       sh_error_message (errno), 
+		       sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		       _("sh_socket_open_int: bind"));
       SL_RETURN( (-1), _("sh_socket_open_int"));
     }
@@ -328,9 +470,9 @@ int sh_socket_open_int ()
   if (0 != setsockopt(pf_unix_fd, SOL_SOCKET, SO_PASSCRED, 
 		      &optval, sizeof(optval)))
     {
-      close(pf_unix_fd); pf_unix_fd = -1;
+      sl_close_fd(FIL__, __LINE__, pf_unix_fd); pf_unix_fd = -1;
       sh_error_handle ((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		       sh_error_message (errno), 
+		       sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		       _("sh_socket_open_int: setsockopt"));
       SL_RETURN( (-1), _("sh_socket_open_int"));
     }
@@ -339,9 +481,9 @@ int sh_socket_open_int ()
   flags = fcntl((pf_unix_fd), F_GETFL);
   if (flags < 0)
     {
-      close(pf_unix_fd); pf_unix_fd = -1;
+      sl_close_fd(FIL__, __LINE__, pf_unix_fd); pf_unix_fd = -1;
       sh_error_handle ((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		       sh_error_message (errno), 
+		       sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		       _("sh_socket_open_int: fcntl1"));
       SL_RETURN( (-1), _("sh_socket_open_int"));
     }
@@ -349,18 +491,18 @@ int sh_socket_open_int ()
   flags = fcntl((pf_unix_fd), F_SETFL, flags|O_NONBLOCK);
   if (flags < 0)
     {
-      close(pf_unix_fd); pf_unix_fd = -1;
+      sl_close_fd(FIL__, __LINE__, pf_unix_fd); pf_unix_fd = -1;
       sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		      sh_error_message (errno), 
+		      sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		      _("sh_socket_open_int: fcntl2"));
       SL_RETURN( (-1), _("sh_socket_open_int"));
     }
 
   if (0 != listen(pf_unix_fd, 5))
     {
-      close(pf_unix_fd); pf_unix_fd = -1;
+      sl_close_fd(FIL__, __LINE__, pf_unix_fd); pf_unix_fd = -1;
       sh_error_handle ((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		       sh_error_message (errno), 
+		       sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		       _("sh_socket_open_int: listen"));
       SL_RETURN( (-1), _("sh_socket_open_int"));
     }
@@ -414,7 +556,8 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   struct socket_cmd * list_cmd;
   char message[SH_MAXMSG];
   struct sockaddr_un name;
-  int size;
+  ACCEPT_TYPE_ARG3 size = sizeof(name);
+
   int nbytes;
   int talkfd;
   int retry = 0;
@@ -423,6 +566,7 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   char * clt = NULL;
 
   int  client_uid = -1;
+  char errbuf[SH_ERRBUF_SIZE];
 
 
   struct msghdr msg;
@@ -438,7 +582,11 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   gid_t peer_gid;
 #elif defined(SO_PEERCRED) 
   struct ucred cr;
-  int          cl = sizeof(cr);
+#ifdef HAVE_SOCKLEN_T
+  socklen_t cl = sizeof(cr);
+#else
+  int       cl = sizeof(cr);
+#endif 
 
 #elif defined(HAVE_STRUCT_CMSGCRED) || defined(HAVE_STRUCT_FCRED) || (defined(HAVE_STRUCT_SOCKCRED) && defined(LOCAL_CREDS))
 
@@ -471,7 +619,7 @@ int sh_socket_read (struct socket_cmd * srvcmd)
     }
 
   iov.iov_base = (char *) &message;
-  iov.iov_len  = SH_MAXMSG;
+  iov.iov_len  = sizeof(message);
 
   memset (&msg, 0, sizeof (msg));
   msg.msg_iov = &iov;
@@ -488,9 +636,10 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   /* the socket is non-blocking 
    * 'name' is the address of the sender socket
    */
-  size = sizeof (name);
-  talkfd = retry_accept(FIL__, __LINE__, 
-			pf_unix_fd, (struct sockaddr *) & name, &size);
+  do {
+    talkfd = accept(pf_unix_fd, (struct sockaddr *) &name, &size);
+  } while (talkfd < 0 && errno == EINTR);
+
   if ((talkfd < 0) && (errno == EAGAIN))
     {
       return 0;
@@ -498,7 +647,7 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   else if (talkfd < 0)
     {
       sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		      sh_error_message (errno), 
+		      sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		      _("sh_socket_read: accept"));
       return -1;
     }
@@ -512,9 +661,9 @@ int sh_socket_read (struct socket_cmd * srvcmd)
     if (setsockopt (talkfd, 0, LOCAL_CREDS, &on, sizeof (on)) < 0)
       {
 	sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-			sh_error_message (errno), 
+			sh_error_message (errno, errbuf, sizeof(errbuf)), 
 			_("sh_socket_read: setsockopt"));
-	close(talkfd);
+	sl_close_fd(FIL__, __LINE__, talkfd);
 	return -1;
       }
   }
@@ -525,15 +674,15 @@ int sh_socket_read (struct socket_cmd * srvcmd)
     if ((nbytes < 0) && (errno != EAGAIN))
       {
 	sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-			sh_error_message (errno),
+			sh_error_message (errno, errbuf, sizeof(errbuf)),
 			_("sh_socket_read: recvmsg"));
-	close(talkfd);	
+	sl_close_fd(FIL__, __LINE__, talkfd);	
 	return -1;
       }
     else if (nbytes < 0)
       {
 	++retry;
-	retry_msleep(0, 1);
+	retry_msleep(0, 10);
       }
   } while ((nbytes < 0) && (retry < 3));
 
@@ -541,22 +690,22 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   fprintf(stderr, "%d bytes received\n", nbytes);
 #endif
 
-  /* nbytes = recv (talkfd, message, SH_MAXMSG, 0); */
-
-  message[SH_MAXMSG-1] = '\0';
+  /* msg.msg_iov.iov_base, filled by recvmsg
+   */
+  message[sizeof(message)-1] = '\0';
 
   if (nbytes < 0)
     {
       if (errno == EAGAIN)
 	{
 	  /* no data */
-	  close(talkfd);
+	  sl_close_fd(FIL__, __LINE__, talkfd);
 	  return 0;
 	}
       sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		      sh_error_message (errno), 
+		      sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		      _("sh_socket_read: recvfrom"));
-      close(talkfd);
+      sl_close_fd(FIL__, __LINE__, talkfd);
       return -1;
     }
 
@@ -564,9 +713,9 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   if (0 != getpeereid(talkfd, &peer_uid, &peer_gid))
     {
       sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		      sh_error_message (errno), 
+		      sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		      _("sh_socket_read: getpeereid"));
-      close(talkfd);
+      sl_close_fd(FIL__, __LINE__, talkfd);
       return -1;
     }
   client_uid = peer_uid;
@@ -575,9 +724,9 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   if (0 != getsockopt(talkfd, SOL_SOCKET, SO_PEERCRED, &cr, &cl))
     {
       sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		      sh_error_message (errno), 
+		      sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		      _("sh_socket_read: getsockopt"));
-      close(talkfd);
+      sl_close_fd(FIL__, __LINE__, talkfd);
       return -1;
     }
   client_uid = cr.uid;
@@ -588,7 +737,51 @@ int sh_socket_read (struct socket_cmd * srvcmd)
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 		      _("Message from recvmsg() was not SCM_CREDS"), 
 		      _("sh_socket_read"));
-      close(talkfd);
+
+      /* Check for file descriptors sent using SCM_RIGHTS, and
+       * close them. If MSG_CTRUNC is set, the buffer was too small,
+       * and no fds are duped.
+       */
+      if (msg.msg_controllen >= sizeof(struct cmsghdr) &&
+	  (msg.msg_flags & MSG_CTRUNC) == 0)
+	{
+	  unsigned int     data_size;
+	  unsigned int     data_i;
+	  int              fdcount, fdmax;
+	  struct cmsghdr * cmptr;
+	  int              fdsbuf[1 + (sizeof(cmsgmem)/sizeof(int))];
+
+	  for (cmptr = CMSG_FIRSTHDR(&msg); cmptr != NULL;
+	       cmptr = CMSG_NXTHDR(&msg, cmptr)) 
+	    {
+	      if (cmptr->cmsg_len > sizeof (cmsgmem) || 
+		  cmptr->cmsg_level != SOL_SOCKET ||
+		  cmptr->cmsg_type  != SCM_RIGHTS)
+		continue;
+
+	      /* Crappy way of finding the data length.
+	       * cmptr->cmsg_len includes both header and padding,
+	       * how are you supposed to find the data length?
+	       * cmptr->cmsg_len - ALIGN(sizeof(struct cmsghdr)) ?
+	       */
+	      data_size = 0;
+
+	      for (data_i = 0; data_i < cmptr->cmsg_len; ++data_i)
+		{
+		  if (CMSG_LEN(data_i) == cmptr->cmsg_len)
+		    {
+		      data_size = data_i;
+		      break;
+		    }
+		}
+	      memcpy(fdsbuf, CMSG_DATA(cmptr), data_size);
+	      fdmax = data_size / sizeof(int);
+	      for (fdcount = 0; fdcount < fdmax; ++fdcount)
+		(void) sl_close_fd(FIL__, __LINE__, fdsbuf[fdcount]);
+	    }
+	}
+      
+      sl_close_fd(FIL__, __LINE__, talkfd);
       return -1;
     }
   cred = (Cred *) CMSG_DATA (cmsg);
@@ -599,7 +792,11 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   eopw = strchr(message, '@');
   if (eopw) 
     *eopw = '\0';
-  if (0 == strcmp(goodpassword, message))
+  /*
+   * message is null-terminated and >> goodpassword
+   */
+  if (0 == strcmp(goodpassword, message) &&
+      strlen(goodpassword) < (sizeof(message)/2))
     {
       client_uid = sh_socket_flaguid;
       cmd = &message[strlen(goodpassword)+1];
@@ -611,14 +808,14 @@ int sh_socket_read (struct socket_cmd * srvcmd)
 		      _("Bad password"), 
 		      _("sh_socket_read"));
       sh_set_sockpass();
-      close(talkfd);
+      sl_close_fd(FIL__, __LINE__, talkfd);
       return -1;
     }
 #else
   sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
 		  _("Socket credentials not supported on this OS"), 
 		  _("sh_socket_read"));
-  close(talkfd);
+  sl_close_fd(FIL__, __LINE__, talkfd);
   return -1;
 #endif
 
@@ -632,7 +829,7 @@ int sh_socket_read (struct socket_cmd * srvcmd)
       sh_error_handle((-1), FIL__, __LINE__, client_uid, MSG_E_SUBGEN,
 		      _("client does not have required uid"), 
 		      _("sh_socket_read: getsockopt"));
-      close(talkfd);
+      sl_close_fd(FIL__, __LINE__, talkfd);
       return -1;
     }
 
@@ -649,23 +846,45 @@ int sh_socket_read (struct socket_cmd * srvcmd)
       *clt = '\0'; ++clt;
       if (sl_strlen(cmd) >= SH_MAXMSGLEN)
 	{
+#ifdef SH_DEBUG_SOCKET
+	  fprintf (stderr, "Server: command too long: %s\n", cmd);
+#endif
 	  sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
 			  _("Bad message format: command too long"), 
 			  _("sh_socket_read"));
-	  close(talkfd);
+	  sl_close_fd(FIL__, __LINE__, talkfd);
 	  return -1;
 	}
       else if (sl_strlen(clt) >= SH_MAXMSGLEN)
 	{
+#ifdef SH_DEBUG_SOCKET
+	  fprintf (stderr, "Server: hostname too long: %s\n", clt);
+#endif
 	  sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
 			  _("Bad message format: hostname too long"), 
 			  _("sh_socket_read"));
-	  close(talkfd);
+	  sl_close_fd(FIL__, __LINE__, talkfd);
 	  return -1;
 	}
       if (cmd[0] == 'L' && cmd[1] == 'I' &&
 	  cmd[2] == 'S' && cmd[3] == 'T')
 	{
+#ifdef SH_DEBUG_SOCKET
+	  fprintf (stderr, "Server: list %s\n", clt);
+#endif
+	  goto list_all;
+	}
+      else if (cmd[0] == 'P' && cmd[1] == 'R' &&
+	  cmd[2] == 'O' && cmd[3] == 'B' && cmd[4] == 'E')
+	{
+#ifdef SH_DEBUG_SOCKET
+	  fprintf (stderr, "Server: probe start %s\n", clt);
+#endif
+	  sh_socket_probe4reload();
+#ifdef SH_DEBUG_SOCKET
+	  fprintf (stderr, "Server: probe done  %s\n", clt);
+#endif
+	  cmd[0] = 'L'; cmd[1] = 'I'; cmd[2] = 'S'; cmd[3] = 'T';cmd[4] = '\0';
 	  goto list_all;
 	}
       sl_strlcpy (srvcmd->cmd, cmd, SH_MAXMSGLEN);
@@ -674,10 +893,13 @@ int sh_socket_read (struct socket_cmd * srvcmd)
     }
   else
     {
+#ifdef SH_DEBUG_SOCKET
+      fprintf (stderr, "Server: bad message\n");
+#endif
       sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
 		      _("Bad message format"), 
 		      _("sh_socket_read"));
-      close(talkfd);
+      sl_close_fd(FIL__, __LINE__, talkfd);
       return -1;
     }
 
@@ -694,11 +916,11 @@ int sh_socket_read (struct socket_cmd * srvcmd)
                        (struct sockaddr *) & name, size);
   */
   nbytes = send (talkfd, cmd, strlen(cmd) + 1, 0);
-  close(talkfd);
+  sl_close_fd(FIL__, __LINE__, talkfd);
   if (nbytes < 0)
     {
       sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-		      sh_error_message (errno), 
+		      sh_error_message (errno, errbuf, sizeof(errbuf)), 
 		      _("sh_socket_read: send"));
       return -1;
     }
@@ -716,7 +938,7 @@ int sh_socket_read (struct socket_cmd * srvcmd)
       list_cmd = runlist;
       while (list_cmd)
 	{
-	  sl_snprintf(message, SH_MAXMSG, _("SENT  %8s  %32s  %s"),
+	  sl_snprintf(message, sizeof(message), _("SENT  %8s  %32s  %s"),
 		      list_cmd->cmd, list_cmd->clt, list_cmd->cti);
 	  /*
 	  sl_strlcpy(message,     _("DONE"), SH_MAXMSG);
@@ -731,9 +953,9 @@ int sh_socket_read (struct socket_cmd * srvcmd)
 	  if (nbytes < 0)
 	    {
 	      sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-			      sh_error_message (errno), 
+			      sh_error_message (errno, errbuf, sizeof(errbuf)), 
 			      _("sh_socket_read: sendto"));
-	      close(talkfd);
+	      sl_close_fd(FIL__, __LINE__, talkfd);
 	      return -1;
 	    }
 	  list_cmd = list_cmd->next;
@@ -743,7 +965,7 @@ int sh_socket_read (struct socket_cmd * srvcmd)
   list_cmd = cmdlist;
   while (list_cmd)
     {
-      sl_snprintf(message, SH_MAXMSG, _(">>>>  %8s  %32s  %s"),
+      sl_snprintf(message, sizeof(message), _(">>>>  %8s  %32s  %s"),
 		  list_cmd->cmd, list_cmd->clt, list_cmd->cti);
       /*
       sl_strlcpy(message,     _(">>>>"), SH_MAXMSG);
@@ -762,9 +984,9 @@ int sh_socket_read (struct socket_cmd * srvcmd)
       if (nbytes < 0)
 	{
 	  sh_error_handle((-1), FIL__, __LINE__, errno, MSG_E_SUBGEN,
-			  sh_error_message (errno), 
+			  sh_error_message (errno, errbuf, sizeof(errbuf)), 
 			  _("sh_socket_read: sendto"));
-	  close(talkfd);
+	  sl_close_fd(FIL__, __LINE__, talkfd);
 	  return -1;
 	}
       list_cmd = list_cmd->next;
@@ -775,7 +997,7 @@ int sh_socket_read (struct socket_cmd * srvcmd)
 		   (struct sockaddr *) & name, size);
   */
   nbytes = send (talkfd, _("END"), 4, 0);
-  close(talkfd);
+  sl_close_fd(FIL__, __LINE__, talkfd);
   return 0;
 }
 /* #if !defined(HAVE_CMSGCRED) || !defined(SO_PEERCRED) */
@@ -786,9 +1008,12 @@ static void sh_socket_add2list (struct socket_cmd * in)
   struct socket_cmd  * new;
 
   new = SH_ALLOC(sizeof(struct socket_cmd));
-  strcpy (new->cmd, in->cmd);
-  strcpy (new->clt, in->clt);
-  strcpy (new->cti, sh_unix_time(0));
+  sl_strlcpy (new->cmd, in->cmd, sizeof(new->cmd));
+  sl_strlcpy (new->clt, in->clt, sizeof(new->clt));
+#ifdef SH_DEBUG_SOCKET
+  fprintf(stderr, "add2list: time set for %s\n", new->clt);
+#endif
+  (void) sh_unix_time(0, new->cti, sizeof(new->cti));
   new->next = cmdlist;
   cmdlist   = new;
 
@@ -804,18 +1029,24 @@ static void sh_socket_add2run (struct socket_cmd * in)
     {
       if (0 == sl_strcmp(new->clt, client_name))
 	{
-	  strcpy (new->cmd, in->cmd);
-	  strcpy (new->clt, in->clt);
-	  strcpy (new->cti, sh_unix_time(0));
+	  sl_strlcpy (new->cmd, in->cmd, sizeof(new->cmd));
+	  sl_strlcpy (new->clt, in->clt, sizeof(new->clt));
+#ifdef SH_DEBUG_SOCKET
+	  fprintf(stderr, "add2run: time reset for %s\n", new->clt);
+#endif
+	  (void) sh_unix_time(0, new->cti, sizeof(new->cti));
 	  return;
 	}
       new = new->next;
     }
 
   new = SH_ALLOC(sizeof(struct socket_cmd));
-  strcpy (new->cmd, in->cmd);
-  strcpy (new->clt, in->clt);
-  strcpy (new->cti, sh_unix_time(0));
+  sl_strlcpy (new->cmd, in->cmd, sizeof(new->cmd));
+  sl_strlcpy (new->clt, in->clt, sizeof(new->clt));
+#ifdef SH_DEBUG_SOCKET
+  fprintf(stderr, "add2run: time set for %s\n", new->clt);
+#endif
+  (void) sh_unix_time(0, new->cti, sizeof(new->cti));
   new->next = runlist;
   runlist   = new;
 
@@ -872,7 +1103,7 @@ int sh_socket_poll()
       return 0;
     }
 
-  sl_strlcpy(cancel_cmd, _("CANCEL"), SH_MAXMSGLEN); 
+  sl_strlcpy(cancel_cmd, _("CANCEL"), sizeof(cancel_cmd)); 
 
   while (sh_socket_read (&cmd) > 0)
     {
@@ -902,7 +1133,7 @@ char * sh_socket_check(const char * client_name)
     {
       if (0 == sl_strcmp(new->clt, client_name))
 	{
-	  sl_strlcpy(out, new->cmd, SH_MAXMSGLEN);
+	  sl_strlcpy(out, new->cmd, sizeof(out));
 	  sh_socket_add2run (new);
 	  sh_socket_rm2list  (client_name);
 	  return out;
